<?php
/**
 * TPC! Memory Usage API
 * 
 * @package TPC_Memory_Usage
 */

function tpc_memory_usage_init() {
	global $tpcmem;
	
	$tpcmem = new tpcmem();
	
	if ( tpc_memory_usage_maybe_display() ) {
		switch ( get_option('tpc_memory_usage_location') ) {	
			case 'header':
				add_action('wp_header', '_tpc_memory_usage');
				add_action('admin_head', '_tpc_memory_usage');
				break;
			
			case 'footer':
				add_action('wp_footer', '_tpc_memory_usage');
				add_action('admin_footer', '_tpc_memory_usage');
				break;
	
			default:
				// Don't display anywhere in comments
				break;
		}
	}
	
	add_action('tpc_memory_usage_log', 'tpc_memory_usage_log');
	add_action('admin_menu', 'tpc_memory_usage_register_config_page');
	add_action('admin_init', 'tpc_memory_usage_admin_init');
}

function tpc_memory_usage_admin_init() {
	global $tpcmem;
	
	if ( !function_exists('plugins_url') ) return;
	
	// Register scripts and styles
	wp_register_script('tpc-memory-usage', plugins_url('tpc-memory-usage/tpc_memory_usage.js'), array('jquery'), $tpcmem->version, true);
	wp_register_style('tpc-memory-usage', plugins_url('tpc-memory-usage/tpc_memory_usage.css'), array(), $tpcmem->version);
	
	// Register hooks and filters
	if ( tpc_memory_usage_maybe_display() ) {
		add_action('wp_dashboard_setup', 'tpc_memory_usage_dashboard');
		add_filter('admin_footer_text', 'tpc_memory_usage_admin_footer');
	}
	
	// Random checkpoints for memory usage (ensures logging still happens even with display elements turned off)
	add_action('setup_theme', 'tpc_memory_usage_checkpoint', 15);
	add_action('save_post', 'tpc_memory_usage_checkpoint', 15);
	add_action('do_robots', 'tpc_memory_usage_checkpoint', 15);
	add_action('wp_head', 'tpc_memory_usage_checkpoint', 15);
	add_action('wp_footer', 'tpc_memory_usage_checkpoint', 15);
	
	add_filter('tpcmem_mb_suffix', 'tpc_bytes_to_mb');
	add_filter('tpcmem_mb_suffix', create_function('$a', '{ return $a . "M"; }'));
	add_filter('tpcmem_mb_percent', 'tpc_bytes_to_mb');
	add_filter('tpcmem_mb_percent', 'tpc_make_percent');

	if ( function_exists('register_setting') ) {
		register_setting('tpc_memory_usage', 'tpc_memory_usage_location');
		register_setting('tpc_memory_usage', 'tpc_memory_usage_graph');
		register_setting('tpc_memory_usage', 'tpc_memory_usage_admin_footer');
		register_setting('tpc_memory_usage', 'tpc_memory_usage_log');
		register_setting('tpc_memory_usage', 'tpc_memory_usage_allowed_users');
	}
}

/**
 * Runs 'tpc_memory_usage_log' action, which is used for updating highest memory use data.
 * 
 * @return void
 */
function tpc_memory_usage_checkpoint() {
	do_action('tpc_memory_usage_log');
}

/**
 * Check if current user should see the memory notices.
 * 
 * @return bool True if allowed, false if not.
 */
function tpc_memory_usage_maybe_display() {
	$allowed_users = get_option('tpc_memory_usage_allowed_users');
	if ( $allowed_users == 'all' || ($allowed_users == 'admin' && current_user_can('manage_options')) )
		return true;
	return false;
}

function tpc_memory_usage_log($newvalue) {
	if ( 'high' != get_option('tpc_memory_usage_log') )
		return;
	
	$current = get_option('tpc_memory_usage_log_highest');
	
	$newvalue = (int) $newvalue;
	$oldvalue = (int) $current['value'];
	
	if ( false === $current || ($newvalue >= $oldvalue) ) {
		$newdata['dt'] = time();
		$newdata['value'] = $newvalue;

		update_option( 'tpc_memory_usage_log_highest', $newdata );
	}
}

/**
 * Retrieve the value for the dashboard usage graph.
 * 
 * @uses get_option() To find out which value to use for the graph.
 * 
 * @return int Either peak or current memory usage.
 */
function tpc_memory_usage_graph_default() {
	global $tpcmem;
	
	if ( 'peak' == get_option('tpc_memory_usage_graph') )
		$mem = $tpcmem->get_peak_usage();
	else
		$mem = $tpcmem->get_usage();
	
	return $mem;
}

//
// Dashboard
//

/**
 * Add the TPC! Memory Usage Dashboard widget.
 * 
 * @uses wp_enqueue_style()
 * @uses wp_add_dashboard_widget()
 */
function tpc_memory_usage_dashboard() {
	wp_enqueue_style('tpc-memory-usage');
	wp_add_dashboard_widget('tpc_memory_usage_dashboard', 'Memory Usage Overview', '_tpc_memory_usage_dashboard');
}

function _tpc_memory_usage_dashboard() {
	global $tpcmem;
	
	add_action('admin_footer', '_tpc_memory_usage_dashboard_js');
	
	$current = $tpcmem->get_usage();
	$peak = $tpcmem->get_peak_usage();
	
	$current_percent = apply_filters( 'tpcmem_mb_percent', $current );
	$peak_percent = apply_filters( 'tpcmem_mb_percent', $peak );
	
	$memory_usage_graph = get_option('tpc_memory_usage_graph');
	if ( 'current' == $memory_usage_graph )
		$gpercent = $current_percent;
	elseif ( 'peak' == $memory_usage_graph )
		$gpercent = $peak_percent;
	else
		$gpercent = false;
	
	$gcolor = '#1c6280';
	$gcolor = $gpercent > 70 ? '#cccc00' : $gcolor;
	$gcolor = $gpercent > 90 ? '#cc0000' : $gcolor;
	
	if ( !empty($gpercent) ) {
		$current_percent_string = " <span style='color:$gcolor;'>($current_percent%)</span>\r";
		$peak_percent_string = " <span style='color:$gcolor;'>($peak_percent%)</span>\r";
	}
	
	
?>

<ul>
	<li><strong><?php esc_html_e('Memory Usage'); ?></strong>: <span><?php echo esc_html( tpc_bytes_to_mb($current) . 'MB' ); ?></span><?php echo $current_percent_string; ?></li>
	<li><strong><?php esc_html_e('Peak Memory Usage'); ?></strong>: <span><?php echo esc_html( tpc_bytes_to_mb($peak) . 'MB' ); ?></span><?php echo $peak_percent_string; ?></li>
	<?php if ( 'high' == get_option('tpc_memory_usage_log') && $highest_usage = get_option('tpc_memory_usage_log_highest') ): ?>
	<li><strong><?php esc_html_e('Highest Memory Usage'); ?></strong>: <span><?php echo esc_html( tpc_bytes_to_mb($highest_usage['value']) . 'MB on ' . date('n/j/y @ g:i a', $highest_usage['dt']) ); ?></span>
	<?php endif; ?>
	<li><strong><?php esc_html_e('WP Memory Limit'); ?></strong>: <span><?php echo WP_MEMORY_LIMIT; ?></span></li>
	<li><strong><?php esc_html_e('PHP Memory Limit'); ?></strong>: <span><?php echo $tpcmem->get_limit(); ?></span></li>
</ul>

<?php if ( false !== $gpercent ): ?>
<div class="progressbar">
<div class="widget" id="tpcmem_pbar_wrap">
<div class="widget" id="tpcmem_pbar" style="width: <?php echo $gpercent; ?>%; background-color: <?php echo $gcolor; ?>;">
	<div id="tpcmem_pbar_percent"><?php echo $gpercent; ?>%</div>
</div><!-- #tpcmem_pbar -->
</div><!-- #tpcmem_pbar_wrap -->
</div><!-- #progressbar -->
<?php endif; ?>

<p id="tpc_memory_usage_more_button" style="font-size: 9px;"><a href="javascript:void(0)">More...</a></p>
<p id="tpc_memory_usage_hide_button" class="hidden" style="font-size: 9px;"><a href="javascript:void(0)">Hide...</a></p>
<div id="tpc_memory_usage_more" class="hidden">
	<ul>
		<li><strong><?php esc_html_e('PHP Version'); ?></strong>: <span><?php echo PHP_VERSION; ?></span></li>
		<li><strong><?php esc_html_e('Server Software'); ?></strong>: <span><?php echo $_SERVER['SERVER_SOFTWARE']; ?></span></li>
		<li><strong><?php esc_html_e('User Agent'); ?></strong>: <span><?php echo $_SERVER['HTTP_USER_AGENT']; ?></span></li>
	</ul>
</div>

<?php
}

/**
 * Displays the dashboard JavaScript file in footer using 'admin_footer' hook.
 * 
 * @uses wp_print_scripts()
 */
function _tpc_memory_usage_dashboard_js() {
	wp_print_scripts( array('tpc-memory-usage') );
}

//
// Configuration
//

/**
 * Register TPC! Memory Usage options page.
 * 
 * @uses add_options_page()
 */
function tpc_memory_usage_register_config_page() {
	$plugin_page = add_options_page(__('TPC! Memory Usage'), 'Memory Usage', 8, 'tpc_memory_usage.php', 'tpc_memory_usage_config_page');
}

function tpc_memory_usage_config_page() {
	global $tpcmem;
	
	// If a POST has occurred, update the options
	if ( isset($_POST['tpc_memory_usage_update']) ) {
		// Update the HTML comment location
		$tpcmem_location = wp_specialchars($_POST['tpc_memory_usage_location']);
		update_option('tpc_memory_usage_location', $tpcmem_location);
		
		// Update the dashboard graph value to use
		$tpcmem_graph_value = wp_specialchars($_POST['tpc_memory_usage_graph']);
		update_option('tpc_memory_usage_graph', $tpcmem_graph_value);
		
		// Update the "display in admin footer" setting
		$tpcmem_admin_footer = (int)$_POST['tpc_memory_usage_admin_footer'];
		update_option('tpc_memory_usage_admin_footer', $tpcmem_admin_footer);
		
		// Update the logging option
		$tpcmem_log = wp_specialchars($_POST['tpc_memory_usage_log']);
		update_option('tpc_memory_usage_log', $tpcmem_log);
		
		// Reset current highest memory usage to zero if checked...
		if ( isset($_POST['tpc_memory_usage_log_reset']) && $_POST['tpc_memory_usage_log_reset'] )
			update_option('tpc_memory_usage_log_highest', array('dt' => time(), 'value' => '0'));
		
		// Update allowed users
		$tpcmem_users = wp_specialchars($_POST['tpc_memory_usage_allowed_users']);
		update_option('tpc_memory_usage_allowed_users', $tpcmem_users);
	}
?>
<div class="wrap">
<?php if ( function_exists('screen_icon') ) screen_icon(); ?>
<h2><?php _e('TPC! Memory Usage Settings'); ?></h2>

<div class="alignright">
	<form action="https://www.paypal.com/cgi-bin/webscr" method="post">
		<input type="hidden" name="cmd" value="_s-xclick">
		<input type="hidden" name="hosted_button_id" value="7666470">
		<input type="image" src="https://www.paypal.com/en_US/i/btn/btn_donate_LG.gif" border="0" name="submit" alt="PayPal - The safer, easier way to pay online!">
		<img alt="If you have found this plugin to be helpful, please show us your support by donating a few dollars." border="0" src="https://www.paypal.com/en_US/i/scr/pixel.gif" width="1" height="1">
	</form>
</div>

<form method="post" action="options-general.php?page=tpc_memory_usage.php">

<h3><?php _e('Basic Settings'); ?></h3>
<table class="form-table" cellspacing="2" cellpadding="5" width="99%">
	<tr>
		<th scope="col" width="30%" valign="top"><label for="tpc_memory_usage_location"><?php esc_html_e('Show memory usage comment in'); ?>:</label></th>
		<td>
			<select name="tpc_memory_usage_location" id="tpc_memory_usage_location">
			<?php tpc_memory_usage_dropdown_display_locations( get_option('tpc_memory_usage_location') ); ?>
			</select>
			
		</td>
	</tr>
	<tr>
		<th valign="top"><label for="tpc_memory_usage_graph"><?php esc_html_e('Value to use for dashboard graph'); ?>:</label></th>
		<td>
			<select name="tpc_memory_usage_graph" id="tpc_memory_usage_graph">
			<?php tpc_memory_usage_dropdown_graph_options( get_option('tpc_memory_usage_graph') ); ?>
			</select>
		</td>
	</tr>
	<tr>
		<th valign="top"><label for="tpc_memory_usage_admin_footer"><?php esc_html_e('Show memory usage in admin footer'); ?></label></th>
		<td>
			<select name="tpc_memory_usage_admin_footer" id="tpc_memory_usage_admin_footer">
			<?php tpc_memory_usage_dropdown_admin_footer( get_option('tpc_memory_usage_admin_footer') ); ?>
			</select>
		</td>
	</tr>
	<tr>
		<th valign="top"><label for="tpc_memory_usage_log"><?php esc_html_e('Logging'); ?></label></th>
		<td>
			<select name="tpc_memory_usage_log" id="tpc_memory_usage_log">
			<?php tpc_memory_usage_dropdown_log_options( get_option('tpc_memory_usage_log') ); ?>
			</select><br/>
			<input type="checkbox" name="tpc_memory_usage_log_reset" id="tpc_memory_usage_log_reset" value="1" /> <label for="tpc_memory_usage_log_reset">Reset current log statistics?</label>
		</td>
	</tr>
	<tr>
		<th valign="top"><label for="tpc_memory_usage_allowed_users"><?php esc_html_e('Allowed users'); ?></label></th>
		<td>
			<select name="tpc_memory_usage_allowed_users" id="tpc_memory_usage_allowed_users">
			<?php tpc_memory_usage_dropdown_allowed_users( get_option('tpc_memory_usage_allowed_users') ); ?>
			</select>
		</td>
	</tr>
</table>

<p class="submit">
	<?php if ( function_exists('settings_fields') ) settings_fields('tpc_memory_usage'); ?>
	<input type="submit" name="tpc_memory_usage_update" value="<?php echo esc_attr('Save Changes'); ?>" />
</p>

</form>

</div>
<?php
}

function tpc_memory_usage_get_display_locations() {
	return array(
		'header' => 'Header',
		'footer' => 'Footer',
		'none' => 'Do not display'
	);
}

function tpc_memory_usage_get_graph_options() {
	return array(
		'current' => 'Current memory usage',
		'peak' => 'Peak memory usage',
		'hide' => 'Do not display'
	);
}

function tpc_memory_usage_get_admin_footer_options() {
	return array(
		0 => 'No',
		1 => 'Yes'
	);
}

function tpc_memory_usage_get_log_options() {
	return array(
		'high' => 'Record highest memory usage',
		'off' => 'Off'
	);
}

function tpc_memory_usage_get_allowed_users() {
	return array(
		'admin' => 'Administrators only',
		'all' => 'Everyone',
		'off' => 'Disabled'
	);
}

//
// Template
//

function _tpc_memory_usage() {
	global $tpcmem;
?>

<!--
TPC! Memory Usage (http://webjawns.com)
<?php echo 'Memory Usage: ' . esc_html( apply_filters('tpcmem_mb_suffix', $tpcmem->get_usage()) ) . "\r"; ?>
<?php echo 'Memory Peak Usage: ' . esc_html( apply_filters('tpcmem_mb_suffix', $tpcmem->get_peak_usage()) ) . "\r"; ?>
<?php echo 'WP Memory Limit: ' . esc_html( WP_MEMORY_LIMIT ) . "\r"; ?>
<?php echo 'PHP Memory Limit: ' . esc_html( $tpcmem->get_limit() ) . "\r"; ?>
-->

<?php
}

function tpc_memory_usage_admin_footer($html) {
	global $tpcmem;
	
	if ( get_option('tpc_memory_usage_admin_footer') )
		$html .= ' | Memory Usage: ' . apply_filters('tpcmem_mb_suffix', tpc_memory_usage_graph_default()) . '/' . $tpcmem->get_lesser_limit('M');
	
	return $html;
}

function tpc_memory_usage_dropdown_display_locations($selected = false) {
	$d = ''; // default
	$l = ''; // other location(s)
	
	$locations = tpc_memory_usage_get_display_locations();
	
	foreach ( (array)$locations as $key => $loc ) {
		$key = esc_attr($key);
		$loc = esc_html($loc);
		
		if ( $key == $selected )
			$d = "<option value='$key' selected='selected'>$loc</option>";
		else
			$l .= "<option value='$key'>$loc</option>";
	}
	
	echo $d . $l;
}

function tpc_memory_usage_dropdown_graph_options($selected = false) {
	$d = ''; // default
	$o = ''; // other options
	
	$options = tpc_memory_usage_get_graph_options();
	
	foreach ( (array)$options as $key => $opt ) {
		$key = esc_attr($key);
		$opt = esc_html($opt);
		
		if ( $key == $selected )
			$d = "<option value='$key' selected='selected'>$opt</option>";
		else
			$o .= "<option value='$key'>$opt</option>";
	}
	
	echo $d . $o;
}

function tpc_memory_usage_dropdown_admin_footer($selected = false) {
	$d = ''; // default
	$o = ''; // other options
	
	$options = tpc_memory_usage_get_admin_footer_options();
	
	foreach ( (array)$options as $key => $opt ) {
		$key = esc_attr($key);
		$opt = esc_html($opt);
		
		if ( $key == $selected )
			$d = "<option value='$key' selected='selected'>$opt</option>";
		else
			$o .= "<option value='$key'>$opt</option>";
	}
	
	echo $d . $o;
}

function tpc_memory_usage_dropdown_log_options($selected = false) {
	$d = ''; // default
	$o = ''; // other options
	
	$options = tpc_memory_usage_get_log_options();
	
	foreach ( (array)$options as $key => $opt ) {
		$key = esc_attr($key);
		$opt = esc_html($opt);
		
		if ( $key == $selected )
			$d = "<option value='$key' selected='selected'>$opt</option>";
		else
			$o .= "<option value='$key'>$opt</option>";
	}
	
	echo $d . $o;
}

function tpc_memory_usage_dropdown_allowed_users($selected = false) {
	$d = ''; // default
	$o = ''; // other options
	
	$options = tpc_memory_usage_get_allowed_users();
	
	foreach ( (array)$options as $key => $opt ) {
		$key = esc_attr($key);
		$opt = esc_html($opt);
		
		if ( $key == $selected )
			$d = "<option value='$key' selected='selected'>$opt</option>";
		else
			$o .= "<option value='$key'>$opt</option>";
	}
	
	echo $d . $o;
}

?>